@@ -37,9 +37,11 @@ Field::Field()
 : m_glom_type(TYPE_INVALID),
   m_visible(true)
 {
+  m_translatable_item_type = TRANSLATABLE_TYPE_FIELD;
 }
 
 Field::Field(const Field& src)
+: TranslatableItem(src)
 {
   //TODO_Performance: Implement this properly, without the extra copy.
   operator=(src);
@@ -51,12 +53,13 @@ Field::~Field()
 
 Field& Field::operator=(const Field& src)
 {
+  TranslatableItem::operator=(src);
+
   m_glom_type = src.m_glom_type;
   m_field_info = src.m_field_info;
   m_data = src.m_data;
 
-  m_strTitle = src.m_strTitle;
-  m_strLookupRelationship = src.m_strLookupRelationship;
+  m_lookup_relationship = src.m_lookup_relationship;
   m_strLookupField = src.m_strLookupField;
 
   m_calculation = src.m_calculation;
@@ -70,21 +73,15 @@ Field& Field::operator=(const Field& src)
 
 bool Field::operator==(const Field& src) const
 {
-  bool bResult = (m_field_info == src.m_field_info);
-  bResult = bResult && (m_glom_type == src.m_glom_type);
-  bResult = bResult && (m_data == src.m_data);
-
-  bResult = bResult && (m_strTitle == src.m_strTitle);
-  bResult = bResult && (m_strLookupRelationship == src.m_strLookupRelationship);
-  bResult = bResult && (m_strLookupField == src.m_strLookupField);
-
-  bResult = bResult && (m_calculation == src.m_calculation);
-
-  bResult = bResult && (m_visible == src.m_visible);
-
-  bResult = bResult && (m_default_formatting == src.m_default_formatting);
-
-  return bResult;
+  return TranslatableItem::operator==(src)
+         && (m_field_info == src.m_field_info)
+         && (m_glom_type == src.m_glom_type)
+         && (m_data == src.m_data)
+         && (m_lookup_relationship == src.m_lookup_relationship)
+         && (m_strLookupField == src.m_strLookupField)
+         && (m_calculation == src.m_calculation)
+         && (m_visible == src.m_visible)
+         && (m_default_formatting == src.m_default_formatting);
 }
 
 bool Field::operator!=(const Field& src) const
@@ -92,6 +89,11 @@ bool Field::operator!=(const Field& src) const
   return !(operator==(src));
 }
 
+Field* Field::clone() const
+{
+  return new Field(*this);
+}
+
 Field::glom_field_type Field::get_glom_type() const
 {
   return m_glom_type;
@@ -125,23 +127,14 @@ void Field::set_data(const Gnome::Gda::Value& data)
   m_data = data;
 }
 
-Glib::ustring Field::get_title() const
+sharedptr<Relationship> Field::get_lookup_relationship() const
 {
-  return m_strTitle;
-}
-void Field::set_title(const Glib::ustring& strTitle)
-{
-  m_strTitle = strTitle;
+  return m_lookup_relationship;
 }
 
-Glib::ustring Field::get_lookup_relationship() const
+void Field::set_lookup_relationship(const sharedptr<Relationship>& relationship)
 {
-  return m_strLookupRelationship;
-}
-
-void Field::set_lookup_relationship(const Glib::ustring& strRelationship)
-{
-  m_strLookupRelationship = strRelationship;
+  m_lookup_relationship = relationship;
 }
 
 Glib::ustring Field::get_lookup_field() const
@@ -157,19 +150,7 @@ void Field::set_lookup_field(const Glib::ustring& strField)
 bool Field::get_is_lookup() const
 {
   //It's a lookup if the lookup relationship and field are set:
-  return ( (m_strLookupRelationship.size() > 0) && (m_strLookupField.size() > 0) );
-}
-
-Glib::ustring Field::get_title_or_name() const
-{
-  if(!m_strTitle.empty())
-  {
-    return m_strTitle;
-  }
-  else
-  {
-    return m_field_info.get_name();
-  }
+  return ( m_lookup_relationship && (!m_strLookupField.empty()) );
 }
 
 namespace { //anonymous
@@ -217,10 +198,42 @@ Glom_PQescapeString(char *to, const char *from, size_t length)
 
 } //anonymous
 
+static std::string glom_escape_text(const std::string src)
+{
+  if(src.empty())
+    return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
+  else
+  {
+    const size_t len = src.size();
+    char* to = (char*)malloc(sizeof(char) * 2 * (len + 1)); //recommended size for to.
+    const size_t len_escaped = Glom_PQescapeString(to, src.c_str(), len);
+    if(!len_escaped)
+    {
+      std::cerr << "glom_escape_text(): Glom_PQescapeString() failed with text: " << src << std::endl;
+
+      if(to)
+        free(to);
+
+      return "''";
+    }
+    else
+    {
+      std::string escaped(to, len_escaped);
+      free(to);
+
+      //Also escape any ";" characters, because these also cause problems, at least with libgda:
+      //See bug #326325.
+      escaped = GlomUtils::string_replace(escaped, ";", "\\073");
+
+      return ("'" + escaped + "'"); //Add single-quotes. Actually escape it 
+      //std::cout << "glom_escape_text: escaped and quoted: " << str << std::endl;
+    }
+  }
+}
+
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
-//g_warning("Field::sql: glom_type=%d", get_glom_type());
-
+  //g_warning("Field::sql: glom_type=%d", get_glom_type());
 
   if(value.is_null())
   {
@@ -263,32 +276,7 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
       {
-        std::string as_string = value.get_string();
-        if(as_string.empty())
-          return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-        else
-        {
-          const size_t len = as_string.size();
-          char* to = (char*)malloc(sizeof(char) * 2 * (len + 1)); //recommended size for to.
-          const size_t len_escaped = Glom_PQescapeString(to, as_string.c_str(), len);
-          if(!len_escaped)
-          {
-            std::cerr << "Glom_PQescapeString() failed with text: " << as_string << std::endl;
-            return "''";
-          }
-          else
-          {
-            std::string escaped(to, len_escaped);
-
-            //Also escape any ";" characters, because these also cause problems, at least with libgda:
-            //See bug #326325.
-            escaped = GlomUtils::string_replace(escaped, ";", "\\073");
-
-            str = "'" + escaped + "'"; //Add single-quotes. Actually escape it 
-            //std::cout << "Field::sql(): escaped and quoted: " << str << std::endl;
-          }
-          free(to);
-        }
+        str = glom_escape_text(value.get_string());
       }
 
       break;
@@ -322,10 +310,20 @@ Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
       if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
       {
         long buffer_size = 0;
-        const gpointer buffer = value.get_binary(buffer_size);  
-        str = "'" + GlomConversions::get_escaped_binary_data((guint8*)buffer, buffer_size) + "'::bytea";
+        const gpointer buffer = value.get_binary(buffer_size);
+        if(buffer && buffer_size)
+        {
+          const std::string escaped_binary_data = std::string((char*)buffer, buffer_size);
+          //Now escape that text (to convert \ to \\, for instance):
+          str = glom_escape_text(escaped_binary_data) /* has quotes */ + "::bytea";
+
+          //Use this when libgda unescapes the binary data internally, as it should: 
+          //str = "'" + GlomConversions::get_escaped_binary_data((guint8*)buffer, buffer_size) + "'::bytea";
+        }
       }
-      
+      else
+        g_warning("Field::sql(): glom_type is TYPE_IMAGE but gda type is not VALUE_TYPE_BINARY");
+
       break;
     }
     default:
@@ -466,7 +464,7 @@ Glib::ustring Field::get_sql_type() const
     {
       g_warning("Field::get_sql_type(): returning unknowntype for field name=%s , glom_type=%d, gda_type=%d", get_name().c_str(), get_glom_type(), m_field_info.get_gdatype());
     }
-    
+
     return strType;
   }
 }
@@ -475,11 +473,11 @@ Glib::ustring Field::get_sql_type() const
 bool Field::field_info_from_database_is_equal(const Gnome::Gda::FieldAttributes& field)
 {
   Gnome::Gda::FieldAttributes temp = m_field_info;
-  
+
   temp.set_auto_increment( field.get_auto_increment() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
   temp.set_default_value( field.get_default_value() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
   temp.set_primary_key( field.get_primary_key() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
-    
+
   return temp == field; 
 }
 
