@@ -39,28 +39,24 @@ public:
 
   bool has_field(const Glib::ustring& field_name) const;
 
-  LayoutItem* add_item(const LayoutItem& item);
-  LayoutItem* add_item(const LayoutItem& item, guint sequence);
+  sharedptr<LayoutItem> add_item(const sharedptr<LayoutItem>& item);
+  sharedptr<LayoutItem> add_item(const sharedptr<LayoutItem>& item, guint sequence);
 //  void add_item(const LayoutGroup& item);
 //  void add_item(const LayoutGroup& item, guint sequence);
 
   virtual void change_field_item_name(const Glib::ustring& table_name, const Glib::ustring& field_name, const Glib::ustring& field_name_new);
   virtual void change_related_field_item_name(const Glib::ustring& table_name, const Glib::ustring& field_name, const Glib::ustring& field_name_new);
-  
-  virtual void change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
-  virtual void change_related_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
 
   void remove_all_items();
-  
-  virtual void debug(guint level = 0) const;
 
-  Glib::ustring m_title;
+  //virtual void debug(guint level = 0) const;
+
   guint m_columns_count;
 
-  typedef std::map<int, LayoutItem*> type_map_items;
+  typedef std::map<int, sharedptr<LayoutItem> > type_map_items;
   type_map_items get_items();
 
-  typedef std::map<int, const LayoutItem*> type_map_const_items;
+  typedef std::map<int, sharedptr<const LayoutItem> > type_map_const_items;
   type_map_const_items get_items() const;
 
   virtual Glib::ustring get_part_type_name() const;
