@@ -83,14 +83,25 @@ public:
   virtual Glib::ustring get_connection_user() const;
   virtual Glib::ustring get_connection_database() const;
 
-  typedef std::vector<Relationship> type_vecRelationships;
+  /** Set the language/locale used by original titles.
+   * Title translations are translations of the text in this language.
+   * @param locale: For instance, "en_US.UTF-8".
+   */
+  void set_translation_original_locale(const Glib::ustring& locale);
+
+  /** Get the language/locale used by original titles.
+   * Title translations are translations of the text in this language.
+   */
+  Glib::ustring get_translation_original_locale() const;
+
+  typedef std::vector< sharedptr<Relationship> > type_vecRelationships;
   virtual type_vecRelationships get_relationships(const Glib::ustring& table_name) const;
   void set_relationships(const Glib::ustring& table_name, const type_vecRelationships& vecRelationships);
 
-  bool get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name, Relationship& relationship) const;
-  void set_relationship(const Glib::ustring& table_name, const Relationship& relationship);
+  sharedptr<Relationship> get_relationship(const Glib::ustring& table_name, const Glib::ustring& relationship_name) const;
+  void set_relationship(const Glib::ustring& table_name, const sharedptr<Relationship>& relationship);
 
-  void remove_relationship(const Relationship& relationship);
+  void remove_relationship(const sharedptr<const Relationship>& relationship);
 
   /** Returns whether the relationship's to-field is a primary key  or unique field, meaning
    * that there can be only one related record for each value of the from-field.
@@ -100,18 +111,17 @@ public:
   /** Returns whether the field is the from-field in a to-one relationship.
    * @see get_relationship_is_to_one(). Ignores hidden tables.
    */
-  bool get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
-  bool get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name, Relationship& relationship) const;
+  sharedptr<Relationship> get_field_used_in_relationship_to_one(const Glib::ustring& table_name, const Glib::ustring& field_name) const;
 
 
-  typedef std::vector<Field> type_vecFields;
+  typedef std::vector< sharedptr<Field> > type_vecFields;
   virtual type_vecFields get_table_fields(const Glib::ustring& table_name) const;
   virtual void set_table_fields(const Glib::ustring& table_name, const type_vecFields& vecFields);
 
-  virtual bool get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName, Field& fieldResult) const;
+  virtual sharedptr<Field> get_field(const Glib::ustring& table_name, const Glib::ustring& strFieldName) const;
 
 
-  typedef std::map<guint, LayoutGroup> type_mapLayoutGroupSequence;
+  typedef std::map<guint, sharedptr<LayoutGroup> > type_mapLayoutGroupSequence;
   type_mapLayoutGroupSequence get_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
 
   void set_data_layout_groups(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name, const type_mapLayoutGroupSequence& groups);
@@ -124,10 +134,14 @@ public:
 
   type_mapLayoutGroupSequence get_data_layout_groups_default(const Glib::ustring& layout_name, const Glib::ustring& parent_table_name) const;
 
-  void fill_layout_field_details(const Glib::ustring& parent_table_name, LayoutGroup& layout_group) const;
+  typedef std::list< sharedptr<TranslatableItem> > type_list_translatables;
+  type_list_translatables get_translatable_layout_items(const Glib::ustring& table_name);
+
+  void fill_layout_field_details(const Glib::ustring& parent_table_name, const sharedptr<LayoutGroup>& layout_group) const;
   void fill_layout_field_details(const Glib::ustring& parent_table_name, type_mapLayoutGroupSequence& sequence) const;
 
 
+
   ///When a field name is changed, change it in the relationships, layouts, reports, and fields data:
   virtual void change_field_name(const Glib::ustring& table_name, const Glib::ustring& strFieldNameOld, const Glib::ustring& strFieldNameNew);
 
@@ -137,10 +151,12 @@ public:
   ///When a relationship name is changed, change it in layouts and reports:
   virtual void change_relationship_name(const Glib::ustring& table_name, const Glib::ustring& name, const Glib::ustring& name_new);
 
-  typedef std::list<TableInfo> type_listTableInfo;
+  typedef std::list< sharedptr<TableInfo> > type_listTableInfo;
   virtual type_listTableInfo get_tables() const;
   virtual void set_tables(const type_listTableInfo& tables);
 
+  sharedptr<TableInfo> get_table(const Glib::ustring table_name) const;
+
   /** Use this after DROPing the table.
    * It removes information about the table, including fields and layouts,
    * and any place that parts of the table are used.
@@ -184,8 +200,8 @@ public:
   type_listReports get_report_names(const Glib::ustring& table_name) const;
   void remove_all_reports(const Glib::ustring& table_name);
 
-  void set_report(const Glib::ustring& table_name, const Report& report);
-  bool get_report(const Glib::ustring& table_name, const Glib::ustring& report_name, Report& report) const;
+  void set_report(const Glib::ustring& table_name, const sharedptr<Report>& report);
+  sharedptr<Report> get_report(const Glib::ustring& table_name, const Glib::ustring& report_name) const;
   void remove_report(const Glib::ustring& table_name, const Glib::ustring& report_name);
 
   void set_layout_record_viewed(const Glib::ustring& table_name, const Glib::ustring& layout_name, const Gnome::Gda::Value& primary_key_value);
@@ -218,31 +234,30 @@ public:
 
   virtual void emit_userlevel_changed();
 
-  /** The LayouItem_Field contains cached full relationship information,
-   *  so we don't have to look up the details so often. This method
-   *  updates that cache. It should be called when the relationship details change.
-   */
-  void update_cached_relationships();
-  void update_cached_relationships(LayoutGroup& group, const Glib::ustring& table_name);
-  void update_cached_relationships(FieldFormatting& formatting, const Glib::ustring& table_name);
-
 protected:
 
   //Overrides:
   virtual bool load_after();
   virtual bool save_before();
 
-  void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, LayoutGroup& group);
-  void save_before_layout_group(xmlpp::Element* node, const LayoutGroup& group);
+  void load_after_layout_group(const xmlpp::Element* node, const Glib::ustring table_name, const sharedptr<LayoutGroup>& group);
+  void save_before_layout_group(xmlpp::Element* node, const sharedptr<const LayoutGroup>& group);
 
-  void load_after_layout_item_field(const xmlpp::Element* element, LayoutItem_Field& item);
-  void load_after_layout_item_field_formatting(const xmlpp::Element* element, FieldFormatting& format, const Field& layout_item);
-  void save_before_layout_item_field(xmlpp::Element* nodeItem, const LayoutItem_Field& item);
-  void save_before_layout_item_field_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, const Field& layout_item);
+  void load_after_layout_item_field(const xmlpp::Element* element, const Glib::ustring& table_name, const sharedptr<LayoutItem_Field>& item);
+  void load_after_layout_item_field_formatting(const xmlpp::Element* element, FieldFormatting& format, Field::glom_field_type field_type, const Glib::ustring& table_name, const Glib::ustring& field_name);
+
+  void load_after_translations(const xmlpp::Element* element, TranslatableItem& item);
+
+  void save_before_layout_item_field(xmlpp::Element* nodeItem, const sharedptr<const LayoutItem_Field>& item);
+  void save_before_layout_item_field_formatting(xmlpp::Element* nodeItem, const FieldFormatting& format, Field::glom_field_type field_type);
+
+  void save_before_translations(xmlpp::Element* nodeItem, const TranslatableItem& item);
 
   void save_changes();
 
-  virtual void on_app_state_userlevel_changed(AppState::userlevels userleve);
+  virtual void on_app_state_userlevel_changed(AppState::userlevels userlevel);
+
+  void fill_translatable_layout_items(const sharedptr<LayoutGroup>& group, type_list_translatables& the_list);
 
   static bool get_node_attribute_value_as_bool(const xmlpp::Element* node, const Glib::ustring& strAttributeName);
   static void set_node_attribute_value_as_bool(xmlpp::Element* node, const Glib::ustring& strAttributeName, bool value = true);
@@ -270,7 +285,12 @@ protected:
   class DocumentTableInfo
   {
   public:
-    TableInfo m_info;
+    DocumentTableInfo()
+    {
+      m_info = sharedptr<TableInfo>(new TableInfo()); //Avoid a null sharedptr.
+    }
+
+    sharedptr<TableInfo> m_info;
 
     type_vecFields m_fields;
     type_vecRelationships m_relationships;
@@ -278,7 +298,7 @@ protected:
     typedef std::list< LayoutInfo > type_layouts;
     type_layouts m_layouts;
 
-    typedef std::map<Glib::ustring, Report> type_reports; //map of report names to reports
+    typedef std::map< Glib::ustring, sharedptr<Report> > type_reports; //map of report names to reports
     type_reports m_reports;
 
     Glib::ustring m_example_rows;
@@ -301,6 +321,7 @@ protected:
   type_map_groups m_groups;
 
   Glib::ustring m_database_title;
+  Glib::ustring m_translation_original_locale;
 
   bool m_block_cache_update; //For efficiency.
   bool m_block_modified_set;
