@@ -375,6 +375,11 @@ void Frame_Glom::alert_no_table()
   }
 }
 
+void Frame_Glom::show_table_refresh()
+{
+  show_table(m_strTableName);
+}
+
 void Frame_Glom::show_table(const Glib::ustring& strTableName)
 {
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
@@ -571,7 +576,7 @@ void Frame_Glom::export_data_to_string(Glib::ustring& the_string, const Glib::us
               row_string += ",";
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
-            row_string += layout_item->m_field.sql(value);
+            row_string += layout_item->get_full_field_details()->sql(value);
 
             //if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             //{
@@ -620,9 +625,9 @@ void Frame_Glom::export_data_to_stream(std::ostream& the_stream, const Glib::ust
               row_string += ",";
 
             //Output data in canonical SQL format, ignoring the user's locale, and ignoring the layout formatting:
-            row_string += layout_item->m_field.sql(value);
+            row_string += layout_item->get_full_field_details()->sql(value);
 
-            if(layout_item->m_field.get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
+            if(layout_item->get_glom_type() == Field::TYPE_IMAGE) //This is too much data.
             {
               if(!GlomConversions::value_is_empty(value))
                 std::cout << "  field name=" << layout_item->get_name() << ", image value not empty=" << std::endl;
@@ -866,31 +871,36 @@ void Frame_Glom::update_table_in_document_from_database()
 
     for(Base_DB::type_vecFields::const_iterator iter = fieldsDatabase.begin(); iter != fieldsDatabase.end(); ++iter)
     {
-      const Field& field_database = *iter;
-
-      //Is the field already in the document?
-      type_vecFields::iterator iterFindDoc = std::find_if( fieldsDocument.begin(), fieldsDocument.end(), predicate_FieldHasName<Field>( field_database.get_name() ) );
-      if(iterFindDoc == fieldsDocument.end()) //If it was not found:
+      sharedptr<Field> field_database = *iter;
+      if(field_database)
       {
-        //Add it
-        fieldsDocument.push_back(field_database);
-        document_must_to_be_updated = true;
-      }
-      else //if it was found.
-      {
-        //Compare the information:
-        Gnome::Gda::FieldAttributes field_info_db = field_database.get_field_info();
-        Field field_document =  *iterFindDoc;
-        if(field_document.field_info_from_database_is_equal( field_info_db )) //ignores auto_increment because libgda does not report it from the database properly.
+        //Is the field already in the document?
+        type_vecFields::iterator iterFindDoc = std::find_if( fieldsDocument.begin(), fieldsDocument.end(), predicate_FieldHasName<Field>( field_database->get_name() ) );
+        if(iterFindDoc == fieldsDocument.end()) //If it was not found:
         {
-          //The database has different information. We assume that the information in the database is newer.
-
-          //Update the field information:
-          field_info_db.set_auto_increment( field_document.get_auto_increment() ); //libgda does not report it from the database properly.
-          iterFindDoc->set_field_info( field_info_db );
-
+          //Add it
+          fieldsDocument.push_back(field_database);
           document_must_to_be_updated = true;
         }
+        else //if it was found.
+        {
+          //Compare the information:
+          Gnome::Gda::FieldAttributes field_info_db = field_database->get_field_info();
+          sharedptr<Field> field_document =  *iterFindDoc;
+          if(field_document)
+          {
+            if(field_document->field_info_from_database_is_equal( field_info_db )) //ignores auto_increment because libgda does not report it from the database properly.
+            {
+              //The database has different information. We assume that the information in the database is newer.
+
+              //Update the field information:
+              field_info_db.set_auto_increment( field_document->get_auto_increment() ); //libgda does not report it from the database properly.
+              (*iterFindDoc)->set_field_info( field_info_db );
+
+              document_must_to_be_updated = true;
+            }
+          }
+        }
       }
     }
 
@@ -899,10 +909,10 @@ void Frame_Glom::update_table_in_document_from_database()
     type_vecFields fieldsActual;
     for(type_vecFields::const_iterator iter = fieldsDocument.begin(); iter != fieldsDocument.end(); ++iter)
     {
-      const Field& field = *iter;
+      sharedptr<Field> field = *iter;
 
       //Check whether it's in the database:
-      type_vecFields::iterator iterFindDatabase = std::find_if( fieldsDatabase.begin(), fieldsDatabase.end(), predicate_FieldHasName<Field>( field.get_name() ) );
+      type_vecFields::iterator iterFindDatabase = std::find_if( fieldsDatabase.begin(), fieldsDatabase.end(), predicate_FieldHasName<Field>( field->get_name() ) );
       if(iterFindDatabase != fieldsDatabase.end()) //If it was found
       {
         fieldsActual.push_back(field);
@@ -1111,9 +1121,8 @@ void Frame_Glom::on_box_reports_selected(const Glib::ustring& report_name)
 {
   m_pDialog_Reports->hide();
 
-  Report report;
-  bool found = get_document()->get_report(m_strTableName, report_name, report);
-  if(found)
+  sharedptr<Report> report = get_document()->get_report(m_strTableName, report_name);
+  if(report)
   {
     m_pDialogLayoutReport->set_report(m_strTableName, report);
     m_pDialogLayoutReport->show();
@@ -1196,6 +1205,8 @@ bool Frame_Glom::connection_request_password_and_choose_new_database_name()
   }
   else
     return false; //The user cancelled.
+
+  return false;
 }
 
 bool Frame_Glom::connection_request_password_and_attempt()
@@ -1369,9 +1380,8 @@ void Frame_Glom::on_menu_report_selected(const Glib::ustring& report_name)
     return;
   }
 
-  Report report;
-  bool found = get_document()->get_report(m_strTableName, report_name, report);
-  if(!found)
+  sharedptr<Report> report = get_document()->get_report(m_strTableName, report_name);
+  if(!report)
     return;
 
   report_build(m_strTableName, report, ""); //TODO: Use found set's where_claus.
@@ -1384,8 +1394,8 @@ void Frame_Glom::on_dialog_layout_report_hide()
   if(true) //m_pDialogLayoutReport->get_modified())
   {
     const Glib::ustring original_name = m_pDialogLayoutReport->get_original_report_name();
-    Report report = m_pDialogLayoutReport->get_report();
-    if(original_name != report.get_name())
+    sharedptr<Report> report = m_pDialogLayoutReport->get_report();
+    if(original_name != report->get_name())
       document->remove_report(m_strTableName, original_name);
 
     document->set_report(m_strTableName, report);
