@@ -169,9 +169,8 @@ void Box_Data_List::on_adddel_user_requested_add()
     {
       index_field_to_edit = index_primary_key;
 
-      Field fieldPrimaryKey;
-      bool test = get_field_primary_key(fieldPrimaryKey);
-      if(test && fieldPrimaryKey.get_auto_increment())
+      sharedptr<Field> fieldPrimaryKey = get_field_primary_key();
+      if(fieldPrimaryKey && fieldPrimaryKey->get_auto_increment())
       {
         //Start editing in the first cell that is not the primary key:
         if(index_primary_key == 0)
@@ -239,21 +238,19 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, gu
 
   Gnome::Gda::Value primary_key_value;
 
-  Field field_primary_key = m_AddDel.get_key_field();
+  sharedptr<Field> field_primary_key = m_AddDel.get_key_field();
 
   //Get the new primary key value, if one is available now:
-  if(field_primary_key.get_auto_increment())
+  if(field_primary_key->get_auto_increment())
   {
     //Auto-increment is awkward (we can't get the last-generated ID) with postgres, so we auto-generate it ourselves;
-    const Glib::ustring& strPrimaryKeyName = field_primary_key.get_name();
+    const Glib::ustring& strPrimaryKeyName = field_primary_key->get_name();
     primary_key_value = generate_next_auto_increment(m_strTableName, strPrimaryKeyName);  //TODO: return a Gnome::Gda::Value of an appropriate type.
   }
   else
   {
     //This only works when the primary key is already stored: primary_key_value = get_primary_key_value(row);
-    LayoutItem_Field layout_item;
-    layout_item.m_field = field_primary_key;
-    primary_key_value = get_entered_field_data(layout_item);
+    primary_key_value = get_entered_field_data_field_only(field_primary_key);
   }
 
   //If no primary key value is available yet, then don't add the record yet:
@@ -271,10 +268,10 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, gu
         //Show the primary key in the row, if the primary key is visible:
 
         //If it's an auto-increment, then get the value and show it:
-        if(field_primary_key.get_auto_increment())
+        if(field_primary_key->get_auto_increment())
         {
-          LayoutItem_Field layout_item;
-          layout_item.m_field = field_primary_key;
+          sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+          layout_item->set_full_field_details(field_primary_key);
           m_AddDel.set_value(row, layout_item, primary_key_value);
         }
 
@@ -300,18 +297,18 @@ void Box_Data_List::on_adddel_user_reordered_columns()
   Document_Glom* pDoc = dynamic_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
-    LayoutGroup group;
-    group.set_name("toplevel");
+    sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
+    group->set_name("toplevel");
 
     AddDel::type_vecStrings vec_field_names = m_AddDel.get_columns_order();
 
     guint index = 0;
     for(AddDel::type_vecStrings::iterator iter = vec_field_names.begin(); iter != vec_field_names.end(); ++iter)
     {
-      LayoutItem_Field layout_item;
-      layout_item.set_name(*iter);
-      layout_item.m_sequence = index;
-      group.add_item(layout_item, index); 
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_name(*iter);
+      layout_item->m_sequence = index;
+      group->add_item(layout_item, index); 
       ++index;
     }
 
@@ -331,13 +328,13 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
     //Just update the record:
     try
     {
-      LayoutItem_Field layout_field = m_AddDel.get_column_field(col);
+      sharedptr<const LayoutItem_Field> layout_field = m_AddDel.get_column_field(col);
 
       Glib::ustring table_name = m_strTableName;
-      Field primary_key_field;
+      sharedptr<Field> primary_key_field;
       Gnome::Gda::Value primary_key_value;
 
-      if(!layout_field.get_has_relationship_name())
+      if(!layout_field->get_has_relationship_name())
       {
         table_name = m_strTableName;
         primary_key_field = m_AddDel.get_key_field();
@@ -347,27 +344,26 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
       {
         //If it's a related field then discover the actual table that it's in,
         //plus how to identify the record in that table.
-        const Glib::ustring relationship_name = layout_field.get_relationship_name();
+        const Glib::ustring relationship_name = layout_field->get_relationship_name();
 
         Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
 
-        Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, relationship_name, relationship);
-        if(test)
+        sharedptr<Relationship> relationship = document->get_relationship(m_strTableName, relationship_name);
+        if(relationship)
         {
-          table_name = relationship.get_to_table();
-          const Glib::ustring to_field_name = relationship.get_to_field();
+          table_name = relationship->get_to_table();
+          const Glib::ustring to_field_name = relationship->get_to_field();
           //Get the key field in the other table (the table that we will change)
-          bool test = get_fields_for_table_one_field(table_name, to_field_name, primary_key_field); //TODO_Performance.
-          if(test)
+          primary_key_field = get_fields_for_table_one_field(table_name, to_field_name); //TODO_Performance.
+          if(primary_key_field)
           {
             //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
-            LayoutItem_Field layout_item;
-            layout_item.set_name(relationship.get_from_field());
+            sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+            layout_item->set_name(relationship->get_from_field());
 
             primary_key_value = get_entered_field_data(layout_item);
 
-            bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
+            const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
             if(!test)
               return;
 
@@ -385,13 +381,13 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
       //Update the field in the record (the record with this primary key):
       const Gnome::Gda::Value field_value = m_AddDel.get_value(row, layout_field);
-      std::cout << "Box_Data_List::on_adddel_user_changed(): field_value = " << field_value.to_string() << std::endl;
-      //const Field& field = layout_field.m_field;
-      //const Glib::ustring strFieldName = layout_field.get_name();
+      //std::cout << "Box_Data_List::on_adddel_user_changed(): field_value = " << field_value.to_string() << std::endl;
+      //const sharedptr<const Field>& field = layout_field->m_field;
+      //const Glib::ustring strFieldName = layout_field->get_name();
 
-      bool bTest = set_field_value_in_database(row, layout_field, field_value, primary_key_field, primary_key_value);
+      const bool bTest = set_field_value_in_database(row, layout_field, field_value, primary_key_field, primary_key_value);
 
-      //Glib::ustring strQuery = "UPDATE " + table_name;
+      //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
       //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
       //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
       //bool bTest = Query_execute(strQuery);
@@ -442,14 +438,14 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
 }
 
-void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_changed, const Gnome::Gda::Value& /* field_value */, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
-  if(field_changed.get_has_relationship_name())
+  if(field_changed->get_has_relationship_name())
     return; //TODO: Handle these too.
 
   //Get values for lookup fields, if this field triggers those relationships:
   //TODO_performance: There is a LOT of iterating and copying here.
-  const Glib::ustring strFieldName = field_changed.get_name();
+  const Glib::ustring strFieldName = field_changed->get_name();
   type_vecLayoutFields fieldsToGet = get_related_fields(strFieldName);
 
   if(!fieldsToGet.empty())
@@ -478,7 +474,7 @@ void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row,
           //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
           //g_warning("  value_as_string=%s", value.to_string().c_str());
 
-          m_AddDel.set_value(row, *layout_item, value);
+          m_AddDel.set_value(row, layout_item, value);
             //g_warning("addedel size=%d", m_AddDel.get_count());
 
           ++iterFields;
@@ -490,26 +486,25 @@ void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row,
   }
 }
 
-void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field_changed, const Gnome::Gda::Value& field_value, const sharedptr<const Field>& primary_key, const Gnome::Gda::Value& primary_key_value)
 {
-   if(field_changed.get_has_relationship_name())
+   if(field_changed->get_has_relationship_name())
     return; //TODO: Handle these too.
 
 
    //Get values for lookup fields, if this field triggers those relationships:
    //TODO_performance: There is a LOT of iterating and copying here.
-   const Glib::ustring strFieldName = field_changed.get_name();
+   const Glib::ustring strFieldName = field_changed->get_name();
    type_list_lookups lookups = get_lookup_fields(strFieldName);
    for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
    {
-     const LayoutItem_Field& layout_Item = iter->first;
+     sharedptr<const LayoutItem_Field> layout_Item = iter->first;
 
-     const Relationship relationship = iter->second;
-     const Field& field_lookup = layout_Item.m_field;
+     sharedptr<const Relationship> relationship = iter->second;
+     const sharedptr<const Field> field_lookup = layout_Item->get_full_field_details();
 
-     Field field_source;
-     bool test = get_fields_for_table_one_field(relationship.get_to_table(), field_lookup.get_lookup_field(), field_source);
-     if(test)
+     sharedptr<const Field> field_source = get_fields_for_table_one_field(relationship->get_to_table(), field_lookup->get_lookup_field());
+     if(field_source)
      {
        Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
 
@@ -518,7 +513,7 @@ void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const Layout
 
        //Add it to the database (even if it is not shown in the view)
        set_field_value_in_database(row, layout_Item, value, primary_key, primary_key_value); //Also does dependent lookups/recalcs.
-       //Glib::ustring strQuery = "UPDATE " + m_strTableName;
+       //Glib::ustring strQuery = "UPDATE \"" + m_strTableName + "\"";
        //strQuery += " SET " + field_lookup.get_name() + " = " + field_lookup.sql(value);
        //strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
        //Query_execute(strQuery);  //TODO: Handle errors
@@ -640,16 +635,16 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value_first()
         return value;
     }
   }
-  
+
   return Gnome::Gda::Value();
 }
 
-Gnome::Gda::Value Box_Data_List::get_entered_field_data(const LayoutItem_Field& field) const
+Gnome::Gda::Value Box_Data_List::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
   return m_AddDel.get_value_selected(field);
 }
 
-void Box_Data_List::set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
+void Box_Data_List::set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
 {
   return m_AddDel.set_value_selected(field, value);
 }
@@ -672,9 +667,8 @@ void Box_Data_List::create_layout()
 
     m_AddDel.set_table_name(m_strTableName);
 
-    Field field_primary_key;
-    bool test = get_field_primary_key_for_table(m_strTableName, field_primary_key);
-    if(!test)
+    sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_strTableName);
+    if(!field_primary_key)
     {
       //g_warning("Box_Data_List::create_layout(): primary key not found.");
     }
@@ -686,9 +680,9 @@ void Box_Data_List::create_layout()
 
       //Add extra possibly-non-visible columns that we need:
       //TODO: Only add it if it is not already there.
-      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
       layout_item->set_hidden();
-      layout_item->m_field = m_AddDel.get_key_field();
+      layout_item->set_full_field_details(m_AddDel.get_key_field());
       m_FieldsShown.push_back(layout_item);
 
 
@@ -699,7 +693,8 @@ void Box_Data_List::create_layout()
         if(m_read_only)
           field->set_editable(false);
 
-        m_AddDel.add_column(*field);
+        //std::cout << "Adding field: name=" << field->get_name() << ", titleorname=" << field->get_title_or_name() << std::endl;
+        m_AddDel.add_column(field);
       }
 
       m_AddDel.set_where_clause(m_strWhereClause);
@@ -746,10 +741,9 @@ bool Box_Data_List::get_field_column_index(const Glib::ustring& field_name, guin
   return false; //failure.
 }
 
-bool Box_Data_List::get_field_primary_key(Field& field) const
+sharedptr<Field> Box_Data_List::get_field_primary_key() const
 {
-  field = m_AddDel.get_key_field();
-  return true;
+  return m_AddDel.get_key_field();
 }
 
 bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
@@ -769,21 +763,21 @@ void Box_Data_List::print_layout()
   else
   {
     //Create a simple report on the fly:
-    Report report_temp;
-    report_temp.m_name = "list";
-    report_temp.m_title = _("List");
+    sharedptr<Report> report_temp(new Report());
+    report_temp->set_name("list");
+    report_temp->set_title(_("List"));
 
     //Add all the fields from the layout:
     for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
     {
-      report_temp.m_layout_group.add_item(*(*iter));
+      report_temp->m_layout_group->add_item(*iter);
     }
 
     report_build(m_strTableName, report_temp, m_strWhereClause);
   }
 }
 
-void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const LayoutGroup& /* group */)
+void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const sharedptr<const LayoutGroup>& /* group */)
 {
 }
 
