@@ -22,14 +22,15 @@
 #define DIALOG_FIELDDEFINITION_H
 
 #include <gtkmm.h>
-#include "combo_textglade.h"
-#include "combo_lookup_relationship.h"
+#include "../../utility_widgets/combo_textglade.h"
+#include "../../combobox_relationship.h"
 #include "combo_fieldtype.h"
 #include "../../utility_widgets/table_columns.h"
-#include "../../utility_widgets/entry_numerical.h"
+//#include "../../utility_widgets/entry_numerical.h"
 #include "../../utility_widgets/dialog_properties.h"
 #include "../../utility_widgets/datawidget.h"
 #include "../../data_structure/field.h"
+#include "../../layout_item_dialogs/box_formatting.h"
 #include "../../base_db.h"
 
 
@@ -41,8 +42,8 @@ public:
   Dialog_FieldDefinition(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_FieldDefinition();
 
-  virtual void set_field(const Field& field, const Glib::ustring& table_name);
-  virtual Field get_field() const;
+  virtual void set_field(const sharedptr<const Field>& field, const Glib::ustring& table_name);
+  virtual sharedptr<Field> get_field() const; //TODO_FieldShared
 
 protected:
 
@@ -52,7 +53,7 @@ protected:
   void on_check_lookup_toggled();
   void on_check_calculate_toggled();
   void on_button_edit_calculation();
-  
+
   //void on_foreach(Gtk::Widget& widget);
 
   //Disable/enable other controls when a control is selected.
@@ -69,19 +70,22 @@ protected:
   Gtk::VBox* m_pBox_DefaultValue;
   Gtk::CheckButton* m_pCheck_Lookup;
   Gtk::Table* m_pTable_Lookup;
-  Combo_LookupRelationship* m_pCombo_LookupRelationship;
+  ComboBox_Relationship* m_pCombo_LookupRelationship;
   Combo_TextGlade* m_pCombo_LookupField;
 
   Gtk::CheckButton* m_pCheck_Calculate;
   Gtk::Alignment* m_pAlignment_Calculate;
   Gtk::TextView* m_pTextView_Calculation;
   Gtk::Button* m_pButton_EditCalculation;
-     
+
   Gtk::Entry* m_pEntry_Title;
 
   DataWidget* m_pDataWidget_DefaultValueSimple;
 
-  Field m_Field;
+  Gtk::VBox* m_box_formatting_placeholder;
+  Box_Formatting* m_box_formatting;
+
+  sharedptr<Field> m_Field;
   Glib::ustring m_table_name;
 };
 
