@@ -161,7 +161,7 @@ void Dialog_GroupsList::set_document(const Glib::ustring& layout, Document_Glom*
     //guint group_sequence = 1; //0 means no sequence
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
     {
-      const LayoutGroup& group = iter->second;
+      sharedptr<const LayoutGroup> group = iter->second;
 
       add_group(Gtk::TreeModel::iterator(), group);
     }
@@ -271,7 +271,7 @@ void Dialog_GroupsList::on_button_group_new()
 
     for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
-      set_table_privileges(group_name, iter->m_name, priv);
+      set_table_privileges(group_name, (*iter)->get_name(), priv);
     }
 
     fill_group_list();
@@ -397,7 +397,7 @@ void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
      // Make sure that these are in the document,
      // so that the correct groups will be created if we recreate the database from the document:
      GroupInfo group_info;
-     group_info.m_name = group_name;
+     group_info.set_name(group_name);
 
     Document_Glom::type_listTableInfo table_list = pDocument->get_tables();
 
@@ -406,7 +406,7 @@ void Dialog_GroupsList::fill_table_list(const Glib::ustring& group_name)
       Gtk::TreeModel::iterator iterTree = m_model_tables->append();
       Gtk::TreeModel::Row row = *iterTree;
 
-      const Glib::ustring table_name = iter->m_name;
+      const Glib::ustring table_name = (*iter)->get_name();
 
       row[m_model_columns_tables.m_col_name] = table_name;
 
