@@ -859,7 +859,7 @@ void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
     {
       //Ask the database how many records there are in the whole table:
       //TODO: Apparently, this is very slow:
-      const Glib::ustring sql_query = "SELECT count(*) FROM " + m_table_name;
+      const Glib::ustring sql_query = "SELECT count(*) FROM \"" + m_table_name + "\"";
       Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->execute_single_command(sql_query);
 
       if(datamodel)
