@@ -133,12 +133,14 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
 
   Glib::ustring sql_part_fields;
 
-  typedef std::list<Relationship> type_list_relationships;
+  typedef std::list< sharedptr<const Relationship> > type_list_relationships;
   type_list_relationships list_relationships;
 
 
   for(type_vecLayoutFields::const_iterator iter =  fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
   {
+    Glib::ustring one_sql_part;
+
     sharedptr<LayoutItem_Field> layout_item = *iter;
 
     bool is_summary = false;
@@ -146,63 +148,78 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
     if(fieldsummary)
       is_summary = true;
 
-    if(iter != fieldsToGet.begin())
-      sql_part_fields += ", ";
-
     //Add, for instance, "SUM(":
     if(is_summary)
-      sql_part_fields += fieldsummary->get_summary_type_sql() + "(";
+      one_sql_part += fieldsummary->get_summary_type_sql() + "(";
 
     if(!layout_item->get_has_relationship_name())
     {
-      sql_part_fields += ( table_name + "." );
+      one_sql_part += ( "\"" + table_name + "\"." );
     }
     else
     {
-      const Glib::ustring relationship_name = layout_item->get_relationship_name();
-      if(!relationship_name.empty())
+      sharedptr<const Relationship> relationship = layout_item->get_relationship();
+
+      /*
+      const Glib::ustring field_table_name = relationship->get_to_table();
+      if(field_table_name.empty())
       {
-        const Relationship relationship = layout_item->m_relationship;
+        g_warning("build_sql_select_with_where_clause(): field_table_name is null. relationship name = %s", relationship->get_name().c_str());
+      }
+      */
 
-        /*
-        const Glib::ustring field_table_name = relationship.get_to_table();
-        if(field_table_name.empty())
-        {
-          g_warning("build_sql_select_with_where_clause(): field_table_name is null. relationship name = %s", relationship.get_name().c_str());
-        }
-        */
+      //We use relationship_name.field_name instead of related_table_name.field_name,
+      //because, in the JOIN below, will specify the relationship_name as an alias for the related table name
+      const Glib::ustring relationship_name = relationship->get_name();
+      one_sql_part += ( "relationship_" + relationship_name + "." );
 
-        //We use relationship_name.field_name instead of related_table_name.field_name,
-        //because, in the JOIN below, will specify the relationship_name as an alias for the related table name
-        sql_part_fields += ( "relationship_" + relationship_name + "." );
+      //Add the relationship to the list:
+      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_FieldHasName<Relationship>( relationship_name ) );
+      if(iterFind == list_relationships.end()) //If the table is not yet in the list:
+        list_relationships.push_back(relationship);
+    }
 
-        //Add the relationship to the list:
-        type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_FieldHasName<Relationship>( relationship_name ) );
-        if(iterFind == list_relationships.end()) //If the table is not yet in the list:
-          list_relationships.push_back(relationship);
+    const Glib::ustring name = layout_item->get_name();
+    if(!name.empty())
+    {
+      one_sql_part += ("\"" + name + "\"");
+
+      //Close the summary bracket if necessary.
+      if(is_summary)
+        one_sql_part +=  ")";
+
+      //Append it to the big string of fields:
+      if(!one_sql_part.empty())
+      {
+        if(!sql_part_fields.empty())
+          sql_part_fields += ", ";
+
+        sql_part_fields += one_sql_part;
       }
     }
 
-    sql_part_fields += layout_item->get_name();
-
-    //Close the summary bracket if necessary.
-    if(is_summary)
-      sql_part_fields +=  ")";
-
   }
 
-  result =  "SELECT " + sql_part_fields +
-    " FROM " + table_name;
+  if(sql_part_fields.empty())
+  {
+    std::cerr << "GlomUtils::build_sql_select_with_where_clause(): sql_part_fields.empty(): fieldsToGet.size()=" << fieldsToGet.size() << std::endl;
+    return result;
+  }
+  else
+  {
+    result =  "SELECT " + sql_part_fields +
+      " FROM \"" + table_name + "\"";
+  }
 
   //LEFT OUTER JOIN will get the field values from the other tables, and give us our fields for this table even if there is no corresponding value in the other table.
   Glib::ustring sql_part_leftouterjoin; 
   for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
   {
-    const Relationship& relationship = *iter;
-    sql_part_leftouterjoin += " LEFT OUTER JOIN " + relationship.get_to_table() + 
-      " AS relationship_" + relationship.get_name() + //Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
-      " ON (" + relationship.get_from_table() + "." + relationship.get_from_field() + " = relationship_" +
-      relationship.get_name() + "." + relationship.get_to_field() +
+    sharedptr<const Relationship> relationship = *iter;
+    sql_part_leftouterjoin += " LEFT OUTER JOIN \"" + relationship->get_to_table() + "\"" + 
+      " AS relationship_" + relationship->get_name() + //Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
+      " ON (\"" + relationship->get_from_table() + "\".\"" + relationship->get_from_field() + "\" = relationship_" +
+      relationship->get_name() + ".\"" + relationship->get_to_field() + "\"" + 
       ")";
   }
 
@@ -218,30 +235,32 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
 }
 
 
-GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const LayoutItem_Field& field)
+GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const sharedptr<const LayoutItem_Field>& field)
 {
   type_list_values_with_second list_values;
 
-  Glib::ustring choice_relationship_name, choice_field, choice_second;
-  field.get_formatting_used().get_choices(choice_relationship_name, choice_field, choice_second);
+  sharedptr<Relationship> choice_relationship;
+  Glib::ustring choice_field, choice_second;
+  field->get_formatting_used().get_choices(choice_relationship, choice_field, choice_second);
+  if(!choice_relationship)
+    return list_values;
 
-  const Relationship relationship = field.get_formatting_used().m_choices_related_relationship;
-  const Glib::ustring to_table = relationship.get_to_table();
+  const Glib::ustring to_table = choice_relationship->get_to_table();
   if(to_table.empty())
   {
-    g_warning("get_choice_values(): table_name is null. relationship name = %s", field.get_formatting_used().m_choices_related_relationship.get_name().c_str());
+    g_warning("get_choice_values(): table_name is null. relationship name = %s", glom_get_sharedptr_name(choice_relationship).c_str());
     return list_values;
   }
 
   const bool with_second = !choice_second.empty();
-  const Glib::ustring sql_second = to_table + "." + choice_second;
+  const Glib::ustring sql_second = "\"" + to_table + "\".\"" + choice_second + "\"";
 
   //Get possible values from database, sorted by the first column.
-  Glib::ustring sql_query = "SELECT " + to_table + "." + choice_field;
+  Glib::ustring sql_query = "SELECT \"" + to_table + "\".\"" + choice_field + "\"";
   if(with_second)
     sql_query += ", " + sql_second;
 
-  sql_query += " FROM " + relationship.get_to_table() + " ORDER BY " + to_table + "." + choice_field;
+  sql_query += " FROM \"" + choice_relationship->get_to_table() + "\" ORDER BY \"" + to_table + "\".\"" + choice_field + "\"";
 
   //Connect to database:
   sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect();
@@ -364,3 +383,54 @@ Glib::ustring GlomUtils::xslt_process(const xmlpp::Document& xml_document, const
 
   return result;
 }
+
+Glib::ustring GlomUtils::string_escape_underscores(const Glib::ustring& text)
+{
+  Glib::ustring result;
+  for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
+  {
+    if(*iter == '_')
+      result += "__";
+    else
+      result += *iter;
+  }
+
+  return result;
+}
+
+/** Get just the first part of a locale, such as de_DE, 
+ * ignoring, for instance, .UTF-8 or @euro at the end.
+ */
+Glib::ustring GlomUtils::locale_simplify(const Glib::ustring& locale_id)
+{
+  Glib::ustring result = locale_id;
+
+  //Get everything before the .:
+  Glib::ustring::size_type posDot = locale_id.find(".");
+  if(posDot != Glib::ustring::npos)
+  {
+    result = result.substr(0, posDot);
+  }
+
+  //Get everything before the @:
+  const Glib::ustring::size_type posAt = locale_id.find("@");
+  if(posAt != Glib::ustring::npos)
+  {
+    result = result.substr(0, posAt);
+  }
+
+  return result;
+}
+
+Glib::ustring GlomUtils::locale_language_id(const Glib::ustring& locale_id)
+{
+  Glib::ustring result;
+
+  const Glib::ustring::size_type posUnderscore = locale_id.find("_");
+  if(posUnderscore != Glib::ustring::npos)
+  {
+    result = locale_id.substr(0, posUnderscore);
+  }
+
+  return result;
+}
